EXPECTED = {'X683': {'extensibility-implied': False,
          'imports': {},
          'object-classes': {},
          'object-sets': {},
          'types': {'IntegerList1': {'members': [{'name': 'elem',
                                                  'type': 'INTEGER'},
                                                 {'name': 'next',
                                                  'optional': True,
                                                  'type': 'IntegerList1'}],
                                     'type': 'SEQUENCE'},
                    'OptionallySignedUTF8String': {'members': [{'name': 'unsigned-data',
                                                                'tag': {'number': 0},
                                                                'type': 'UTF8String'},
                                                               {'name': 'signed-data',
                                                                'tag': {'number': 1},
                                                                'type': 'SEQUENCE',
                                                                'members': [
                                                                    {'name': 'authenticated-data',
                                                                     'type': 'UTF8String'},
                                                                    {'name': 'authenticator',
                                                                     'type': 'BIT STRING'}]}],
                                                   'type': 'CHOICE'},
                    'SignedUTF8String': {'members': [{'name': 'authenticated-data',
                                                      'type': 'UTF8String'},
                                                     {'name': 'authenticator',
                                                      'type': 'BIT STRING'}],
                                         'type': 'SEQUENCE'}},
          'values': {}}}
