EXPECTED = {'Foo': {'extensibility-implied': False,
         'imports': {},
         'object-classes': {},
         'object-sets': {},
         'types': {'A': {'type': 'INTEGER'}, 'C': {'type': 'OCTET STRING'}},
         'values': {}}}