EXPECTED = {'Foo': {'extensibility-implied': False,
         'imports': {},
         'object-classes': {},
         'object-sets': {},
         'types': {'B': {'type': 'DATE'},
                   'C': {'type': 'TIME-OF-DAY'},
                   'D': {'type': 'DATE-TIME'}},
         'values': {}}}
